@@ -23,6 +23,7 @@
 
 #include <gst/gst.h>
 #include <gst/base/gstadapter.h>
+#include <gst/base/gstflowcombiner.h>
 
 #include "asfheaders.h"
 
@@ -44,6 +45,7 @@ GST_DEBUG_CATEGORY_EXTERN (asfdemux_dbg);
 
 typedef struct _GstASFDemux GstASFDemux;
 typedef struct _GstASFDemuxClass GstASFDemuxClass;
+typedef enum _GstASF3DMode GstASF3DMode;
 
 typedef struct {
   guint32	packet;
@@ -56,6 +58,21 @@ typedef struct {
   guint16                 len;      /* save this so we can skip unknown IDs  */
 } AsfPayloadExtension;
 
+/**
+ * 3D Types for Media play
+ */
+enum _GstASF3DMode
+{
+  GST_ASF_3D_NONE = 0x00,
+
+  //added, interim format - half
+  GST_ASF_3D_SIDE_BY_SIDE_HALF_LR = 0x01,
+  GST_ASF_3D_SIDE_BY_SIDE_HALF_RL = 0x02,
+  GST_ASF_3D_TOP_AND_BOTTOM_HALF_LR = 0x03,
+  GST_ASF_3D_TOP_AND_BOTTOM_HALF_RL = 0x04,
+  GST_ASF_3D_DUAL_STREAM = 0x0D,                  /**< Full format*/
+};
+
 typedef struct
 {
   gboolean        valid;               /* TRUE if structure is valid/filled */
@@ -95,9 +112,12 @@ typedef struct
 
   GstCaps    *caps;
 
+  GstBuffer *streamheader;
+
   GstTagList *pending_tags;
 
   gboolean    discont;
+  gboolean    first_buffer;
 
   /* Descrambler settings */
   guint8               span;
@@ -106,7 +126,6 @@ typedef struct
   guint16              ds_data_size;
 
   /* for new parsing code */
-  GstFlowReturn   last_flow; /* last flow return */
   GArray         *payloads;  /* pending payloads */
 
   /* Video stream PAR & interlacing */
@@ -168,6 +187,7 @@ struct _GstASFDemux {
   guint32              num_streams;
   AsfStream            stream[GST_ASF_DEMUX_NUM_STREAMS];
   gboolean             activated_streams;
+  GstFlowCombiner     *flowcombiner;
 
   /* for chained asf handling, we need to hold the old asf streams until
    * we detect the new ones */
@@ -205,6 +225,9 @@ struct _GstASFDemux {
   AsfSimpleIndexEntry *sidx_entries;     /* packet number for each entry   */
   
   GSList              *other_streams;    /* remember streams that are in header but have unknown type */
+
+  /* parsing 3D */
+  GstASF3DMode asf_3D_mode;
 };
 
 struct _GstASFDemuxClass {
